/*
** Module   : DropDownGuiImpl.java
** Abstract : Drop down box for combo-box GUI specific implementation.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20150428 Created initial version.
** 002 EVL 20150514 Adding method to adjust the visible rows value according to GUI needs.
**                  The scrollbar approach changed from ALWAYS to default(AS_NEEDED).  Several UI
**                  painting related fixes.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.ui.client.ScrollBar.Bars;

/**
 * Implements the drop-down window for a combo-box in GUI mode.
 * TODO<SIY>: this class is not managed by layout manager, review it and make
 * sure that it conforms GUI requirement, i.e. properly maintains its physical 
 * position/size. 
 */
public class DropDownGuiImpl
extends DropDown<GuiOutputManager>
{
   /** Inner lines minimum. */   
   public static final int INNER_LINES_MIN  = 3;
   
   /** Flag to conrol whether the drop-dow be painting or not. */
   private boolean drawEnabled = false;
   
   /**
    * Create a drop-down container based on the parent combo-box and it's
    * associated data model.
    * 
    * @param    parent
    *           The combo-box object which calls this drop-down.
    */
   @SuppressWarnings("unchecked")
   public DropDownGuiImpl(ComboBox<GuiOutputManager> parent)
   {
      super(parent);
      // disable layout manager
      setLayout(null);
   }

   /**
    * Calculate (and set) the height and position of the drop-down
    * container based on the parent combo-box and it's associated data 
    * model.
    * <p>
    * Precedence order (in the following list the term "max-space" is
    * the number of rows from the top of the containing combo-box's
    * frame to the bottom of the screen) for vertical size:
    * <p>
    * <pre>
    * 1. If there is an inner-lines definition AND it is smaller than the 
    *    max-space, then it is honored.
    * 2. If the number of elements in the data model is less than the
    *    number of rows in the max-space, then the number of data model
    *    elements will determine the vertical size.
    * 3. max-space is used otherwise.
    * </pre>
    * <p>
    * The drop-down box X coordinate is always the same as the X
    * coordinate of the parent combo-box in absolute screen units. For
    * the Y coordinate, the following positioning algorithm is used:
    * <p>
    * <pre>
    * 1. If the height of the drop-down is less than or equal to the space
    *    between the bottom of the combo-box and the bottom of the screen,
    *    then the drop-down T coordinate will be set to the combo-box Y +
    *    1.
    * 2. Otherwise, the drop-down Y coordinate will equal the Y coordinate
    *    of the bottom of the screen minus the height of the drop-down.
    *    This will result in the drop-down ending at the bottom of the
    *    screen and starting at or above the combo-box Y coordinate BUT
    *    never above the top of the frame containing the combo-box
    *    (because of the previously constrained height calculation above).
    * </pre>
    * 
    * @param    parent
    *           The combo-box object which calls this drop-down.
    * @param    model
    *           The data model to be used within this drop-down list.
    */
   @Override
   public void setHeightAndPos(ComboBox<GuiOutputManager> parent, ComboBoxModel<String> model)
   {
      // in GUI there is no frame border insets around drop-down
      setInsets(new Insets(0, 0, 0, 0));
      // call ChUI version of the method first
      super.setHeightAndPos(parent, model);
      
      // start GUI specifics
      // get the coord of the combo-box this drop-down belongs to
      NativePoint ddNative = parent.physicalLocation();
      
      // Get the physical location of the frame containing the drop-down
      FrameGuiImpl fr = (FrameGuiImpl) UiUtils.locateFrame(parent);
      NativePoint frNative = fr.physicalLocation();
      
      // set the coordinates relative to parent combo-box widget
      //
      // The combo-box and drop-down both belong to different frames. The parent frame
      // for drop-down is border-less and header-less invisible frame. So when we get the physical
      // location of the combo-box we have location related to frame the combo-box belongs to. If
      // we use this coords to place drop-down we have XY shift according to combo-box's frame
      // decoration. That's why we can not just use combo-box location plus combo box height here
      // and need to know this info about fra,e decorations.
      //
      ddNative.x += frNative.x;
      // check if the combo-box encrosing frame has a frane - heed one pixel to shift more 
      if (fr.config().box)
      {
         ddNative.x += 1;
      }
      // add combo-box's frame header and combo-box height plus two pixel for combo-box border
      // and one pixel more to have pixel distance to combo-box
      ddNative.y += screen().coordinates().heightToNative(fr.captionHeight()) +
                    screen().coordinates().heightToNative(parent.height()) + 3;

      // set srollpane location relative to main combo-box part
      ScrollPaneGuiImpl scrollPaneGui = (ScrollPaneGuiImpl) scrollpane;
      scrollPaneGui.setPhysicalLocation(ddNative.x, ddNative.y);
      // fix up drop-down location
      // location in character units
      setLocation(screen().coordinates().pointFromNative(frNative));
      // location in pixels
      setPhysicalLocation(frNative.x, frNative.y);
      
      // init scrollbars, visible on demand
      scrollPaneGui.setScrollBars(Bars.VERTICAL);
      scrollPaneGui.setBoxFlag(true);
         
      ((ScrollableSelectionListGuiImpl)scrollPaneGui.getChild()).
                                       setScrollBar(scrollPaneGui.vbar());
   }
   
   /**
    * Get parent container.
    * 
    * @return  parent container object.
    */
   @Override
   public Container<GuiOutputManager> parent()
   {
      // this is the protection for exception cause by detecting the drop down ancestor
      // in the process of the drop-down closing when it is under destruction
      return super.parent() == null ? getComboBox().parent() : super.parent();
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      return getComboBox().width();
   }

   /**
    * Determine the physical bounds, relative to screen, expressed in units native to UI type.
    * 
    * @return  See above.
    */
   @Override
   public NativeRectangle physicalBounds()
   {
      return new NativeRectangle(scrollpane.physicalLocation(), physicalDimension());
   }

   /**
    * Draw widget.  Filter superclass for special cases.
    */
   @Override
   public void draw()
   {
      // drawaing is enabling late enough to prevent screen distortion 
      if (drawEnabled)
      {
         super.draw();
      }
   }

   /**
    * Enables drawing for this drop-down.  This is ideally the last call before starting event
    * processing loop.
    */
   public void enableDraw()
   {
      drawEnabled = true;
   }
   
   /**
    * Get the combo-box GUI specific visible row adjust.
    * 
    * @param    visibleRows
    *           The visible rows value to check and modify.
    * @param    listSize
    *           The total size of the items in the list to handle.
    * 
    * @return  the new visible rows value.
    */
   @Override
   protected int fixVisibleRows(int visibleRows, int listSize)
   {
      int rowsToReturn = visibleRows;
     
      // check the items size
      if (rowsToReturn > listSize)
      {
         rowsToReturn = listSize;
      }
      // but there is a minimum for inner lines 
      if (rowsToReturn < INNER_LINES_MIN)
      {
         rowsToReturn = INNER_LINES_MIN;
      }
      
      return rowsToReturn;
   }
}
